/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2003 Matt Carter
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 */
#ifndef OAPR_EXC_APR_HPP
#define OAPR_EXC_APR_HPP

#include "oapr.hpp"
#include "oapr/oapr_buildconfig.hpp"
#include "apr_errno.h"
#include "oapr_exc_base.hpp"
#include <string>

FILE_VERSION("$Revision: 1.3 $",oapr_exc_apr_hpp);

#define APR_STRERROR_BUFFER_SIZE 1024

namespace oapr
{

  /**
   * Checks the result of an APR operation, and throws an exception if the code is not APR_SUCCESS.
   * @param code Return code of operation.
   * @remark Use this only when you expect the operation to succeed and it is an exceptional event
   * for it to fail (e.g. out of memory, file i/o error). Expected status codes should be
   * handled explicitly.
   */

  class LIBOAPR_API oapr_exception: public exception_base
  {
  public:
    oapr_exception::oapr_exception(apr_status_t code, std::string function, int level = 0)
      : exception_base(0, function, "", "", level),
        m_code(code)
    {
      char buf[APR_STRERROR_BUFFER_SIZE];
      message(apr_strerror(m_code, buf, sizeof(buf)));
    }

    oapr_exception(int line, std::string function, std::string file,
                   apr_status_t code, int level = 0)
      : exception_base(line, function, file, std::string(), level),
        m_code(code)
    {
      char buf[APR_STRERROR_BUFFER_SIZE];
      message(apr_strerror(m_code, buf, sizeof(buf)));
    }

    oapr_exception(int line, std::string function, std::string file,
                   apr_status_t code, std::string msg, int level = 0)
      : exception_base(line, function, file, msg, level),
        m_code(code)
    {}

    oapr_exception(const oapr_exception& oe, int line, std::string func,
                   std::string file, std::string src,
                   std::string msg, int level)
      : exception_base(oe,line,func,file,src,msg,level),
        m_code(oe.m_code)
    {}

    virtual ~oapr_exception() throw() {}
    apr_status_t code() const {return m_code;}
  private:
    apr_status_t m_code;
  };

  inline void apr_check(int line, const char *function, const char *file,
                 apr_status_t code, int level = 0)
  {
    if(code != 0) throw oapr_exception(line, function, file, code);
  }

  inline void apr_check(int line, const char *function, const char *file,
                 apr_status_t code, std::string msg, int level = 0)
  {
    if(code != 0) throw oapr_exception(line, function, file, code, msg);
  }

  inline void apr_check(apr_status_t code)
  {
    if(code != 0) throw oapr_exception(code, "apr_check");
  }

};

#endif
