/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 Justin Kirby
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 */
#include "oapr/oapr_buildconfig.hpp"
#include "oapr/oapr_io_file.hpp"
#include "oapr/oapr_exception.hpp"
#include "oapr/oapr_exc_utils.hpp"
#include "oapr/oapr_pool_mgr.hpp"

FILE_VERSION("$Revision: 1.8.2.1 $",oapr_io_file_cpp);

namespace oapr
{
  apr_status_t basic_file::open(const std::string& addr, apr_int32_t flags, apr_int32_t perms)
  {
    if(is_open()){
      OAPR_THROW(logic_error,"already has an open file, close first");
    }
    apr_status_t rv;
    m_fileName=addr;
    rv=apr_file_open(&m_file,m_fileName.c_str(),flags,perms,m_pool->getp());
    if(rv==APR_SUCCESS){
      m_isOpen=true;
    }
    return rv;
  }

  apr_status_t basic_file::close()
  {
    /**\todo do we need to check for open?*/
    apr_status_t rv;
    if((rv=apr_file_close(m_file))==APR_SUCCESS){
      m_isOpen=false;
    }
    return rv;
  }
  bool basic_file::eof()
  {
    if(apr_file_eof(m_file) == APR_EOF){
      return true;
    }else{
      return false;
    }
  }

  apr_size_t basic_file::write(const void* buff, apr_size_t toWrite, bool full)  {
    if(!is_open()){
      /**\todo throw evil things*/
      return 0;
    }
    apr_size_t rv(toWrite);
    if(full){
      OAPR_CHECK(apr_file_write_full(m_file,buff,toWrite,&rv));
    }else{
      OAPR_CHECK(apr_file_write(m_file,buff,&rv));
    }
    return rv;
  }
  apr_size_t basic_file::read(void* buff, apr_size_t toRead, bool full)const
  {
    if(!is_open()){
      /**\todo throw evil things*/
      return 0;
    }
    apr_size_t rv(toRead);
    if(full){
      OAPR_CHECK(apr_file_read_full(m_file,buff,toRead,&rv));
    }else{
      apr_status_t rc = apr_file_read(m_file,buff,&rv);
      // debug - see mc
      if(rc != APR_SUCCESS)
      {
        if(rc == APR_EOF)
        {
          assert(rv == 0);
          return 0;
        }
        else throw oapr_exception(rc, OAPR_FUNCTION);
      }
    }
    return rv;
  }

  apr_status_t basic_file::flush()
  {
    return apr_file_flush(m_file);
  }


  apr_status_t remove(const basic_file& f)
  {
    return apr_file_remove(f.name().c_str(),f.gpool()->getp());
  }
  apr_status_t remove(const std::string &nm)
  {
    pool::pool_ptr pl=pool_mgr::instance()->get("file");
    return apr_file_remove(nm.c_str(),pl->getp());
  }
  apr_status_t rename(const basic_file& f,const std::string& to)
  {
    return apr_file_rename(f.name().c_str(),to.c_str(),f.gpool()->getp());
  }
  apr_status_t rename(const std::string& from,const std::string& to)
  {
    pool::pool_ptr pl=pool_mgr::instance()->get("file");
    return apr_file_rename(from.c_str(),to.c_str(),pl->getp());
  }
  apr_status_t copy(const basic_file& f,const std::string& to,
		    apr_fileperms_t perms)
  {
    return apr_file_copy(f.name().c_str(),to.c_str(),perms,f.gpool()->getp());
  }
  apr_status_t copy(const std::string& from,const std::string& to,
		    apr_fileperms_t perms)
  {
    pool::pool_ptr pl=pool_mgr::instance()->get("file");
    return apr_file_copy(from.c_str(),to.c_str(),perms,pl->getp());
  }
  apr_status_t append(const basic_file& f,const std::string& to,
				  apr_fileperms_t perms)
  {
    return apr_file_append(f.name().c_str(),to.c_str(),perms,f.gpool()->getp());
  }
  apr_status_t append(const std::string& from,const std::string& to,
		      apr_fileperms_t perms)
  {
    pool::pool_ptr pl=pool_mgr::instance()->get("file");
    return apr_file_append(from.c_str(),to.c_str(),perms,pl->getp());
  }

};
