/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2003 Matt Carter. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGEAPR_UNHANDLED_EXCEPTIONNCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 */

#include "oapr/oapr.hpp"
#include "oapr/oapr_buildconfig.hpp"
#include "oapr/oapr_thread.hpp"
#include "oapr/oapr_exception.hpp"
#include "oapr/oapr_errno.hpp"
#include <assert.h>
#include <stdlib.h>

FILE_VERSION("$Revision: 1.6 $",oapr_thread_cpp);

using namespace oapr;

/**\todo There are conditions where apr thread exit or join functions fail and there is nothing we can
  * do to progress the thread to termination safely. The current course of action is to exit the process
  * as any other action would leave the program in an inconsistent state in which it cannot shut down.
  * Look at this. Perhaps we can skip joining the failed thread, emulate a stable state, and allow as much
  * program cleanup as possible to take place. See all occurrences of FATAL_ERROR. */

/**\todo Thread cannot be restarted yet. Decide if this is desirable and state one or the other. */

#define FATAL_ERROR ::exit(EXIT_FAILURE);

thread_class::thread_class(const pool_ptr& p)
  : m_state(tsSuspended),
    m_state_lock(p),
    m_thread(NULL),
    m_terminated(p),
    m_pool(p)
{
}

thread_class::~thread_class()
{
}

void thread_class::initialize()
{
}

void thread_class::destroy()
{
  try
  {
    mlock lock(m_state_lock);
    switch(m_state)
    {
      case tsSuspended:
      case tsTerminated:
        // Nothing to do
        return;
      case tsRunning:
        // Signal terminate
        OAPR_CHECK(do_terminate());
      case tsTerminating:
        // Wait for our termination condition
        m_terminated.wait(m_state_lock);
        // Wait for thread to completely exit
        apr_status_t exitcode;
        if(apr_thread_join(&exitcode, m_thread) != APR_SUCCESS) FATAL_ERROR
    }
  } catch(...) { FATAL_ERROR }
}

void thread_class::start()
{
  mlock lock(m_state_lock);
  if(!(m_state == tsSuspended || m_state == tsTerminated)) OAPR_THROW(logic_error, "Cannot start already active thread");
  assert(m_thread == NULL);
  OAPR_CHECK(apr_thread_create(&m_thread, NULL, thread_entry_fn, (void *)(this), m_pool->getp()));
}

void* thread_class::thread_main(apr_thread_t *thd) throw()
{
  apr_status_t exitcode;
  if(m_thread != thd)
  {
    mlock lock(m_state_lock);
    m_state = tsTerminated;
    m_exitcode = APR_INTERNAL_THREAD_LOGIC_ERROR;
    return NULL;
  }
  try
  {
    // Set running state
    {
      mlock lock(m_state_lock);
      m_state = tsRunning;
    }
    // Execute thread
    exitcode = execute();
  }
  catch(exception_base& e)
  {
    cleanup_exception(&e);
    return NULL;
  }
  catch(...)
  {
    cleanup_exception(NULL);
    return NULL;
  }
  // Set terminated state
  cleanup(exitcode);
  return NULL;
}

void thread_class::cleanup(apr_status_t exitcode) throw()
{
  try
  {
    // If this lock fails, we have no safe way to progress the thread state.
    mlock lock(m_state_lock);
    m_state = tsTerminated;
    m_exitcode = exitcode;
    m_terminated.signal();
  }
  catch(...) { FATAL_ERROR }
  if(apr_thread_exit(m_thread, exitcode) != APR_SUCCESS) { FATAL_ERROR };
}

void thread_class::cleanup_exception(exception_base *e) throw()
{
  apr_thread_t *thd = m_thread;
  try
  {
    // If this lock fails, we have no safe way to progress the thread state.
    mlock lock(m_state_lock);
    m_state = tsTerminated;
    m_exitcode = unhandled_exception(e);
    m_terminated.signal();
  }
  catch(...) { FATAL_ERROR }
  if(apr_thread_exit(thd, m_exitcode) != APR_SUCCESS) { FATAL_ERROR };
}

void* APR_THREAD_FUNC thread_class::thread_entry_fn(apr_thread_t *thd, void *data)
{
  return static_cast<thread_class*>(data)->thread_main(thd);
}

apr_status_t thread_class::unhandled_exception(exception_base *base) throw()
{
  // Base class implementation does nothing much.
  return APR_EUNCAUGHT_EXCEPTION;
}

void thread_class::exit(apr_status_t status)
{
  apr_thread_t *thd = m_thread;
  assert(thd);
  cleanup(status); // Sets terminated state and m_exitcode. m_thread is now NULL.
}

void thread_class::signal_terminate()
{
  mlock lock(m_state_lock);
  if(m_state == tsSuspended) OAPR_THROW(logic_error, "Cannot terminate thread that hasn't started");
  OAPR_CHECK(do_terminate());
}

apr_status_t thread_class::terminate()
{
  // Tell the thread to terminate
  signal_terminate();
  // Join it
  return join();
}

apr_status_t thread_class::do_terminate()
{
  m_state = tsTerminating;
  return APR_SUCCESS;
}

apr_status_t thread_class::join()
{
  // We use our own join mechanism, as join in pthreads/apr seems to have unreliable behaviour
  // between threads that are running and threads that are not. Also join cleans up
  // resources so will only return success on the first call. Weird!
  // So, we use join on thread destruction and implement our own for general joining
  // purposes.
  mlock lock(m_state_lock);
  if(m_state == tsSuspended) OAPR_THROW(logic_error, "Thread must be started to join()");
  if(m_state == tsRunning || m_state == tsTerminating) m_terminated.wait(m_state_lock);
  return m_exitcode;
}

bool thread_class::active()
{
  mlock lock(m_state_lock);
  return (m_state == tsRunning) || (m_state == tsTerminating);
}

bool thread_class::terminated()
{
  mlock lock(m_state_lock);
  return (m_state == tsTerminating) || (m_state == tsTerminated);
}

bool thread_class::suspended()
{
  mlock lock(m_state_lock);
  return m_state == tsSuspended;
}

thread_state thread_class::state()
{
  mlock lock(m_state_lock);
  return m_state;
}
