/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2003 Matt Carter. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

#include <iostream>
#include "oapr/oapr_app.hpp"
#include "oapr/oapr_exception.hpp"
#include "oapr/oapr_exc_utils.hpp"
#include "oapx/oapr_financial.hpp"
#include <sstream>

using namespace std;
using namespace oapr;

bool check_monetary(const char *amt, int expected)
{
    money amount(amt);
    if(amount.pennies() != expected) {
      cout << "Failed test for parsing monetary amount " << amt << ": expected " << expected << " pennies, got " << amount.pennies() << ".\n";
      return true;
    }
    cout << "Passed test for parsing monetary amount " << amt << ": expected " << expected << " pennies, got " << amount.pennies() << ".\n";
    return false;
}

bool check_func(int result, int expected, const char *desc)
{
  if(result != expected) {
    cout << "Failed function test " << desc << ": expected " << expected << ", got " << result << ".\n";
    return true;
  }
  cout << "Passed function test " << desc << ": expected " << expected << ", got " << result << ".\n";
  return false;
}

bool check_money(money money_result, int expected, const char *desc)
{
  int result = money_result.pennies();
  if(result != expected) {
    cout << "Failed function test " << desc << ": expected " << expected << ", got " << result << ".\n";
    return true;
  }
  cout << "Passed function test " << desc << ": expected " << expected << ", got " << result << ".\n";
  return false;
}

bool check_string(const string& result, const char *expected, const char *desc)
{
  if(result != expected) {
    cout << "Failed function test " << desc << ": expected \"" << expected << "\", got \"" << result << "\".\n";
    return true;
  }
  cout << "Passed function test " << desc << ": expected \"" << expected << "\", got \"" << result << "\".\n";
  return false;
}

int main()
{
  try
  {
    oapr::oapr_app app("oapx financial utils test");
    
    bool failed = false;
    
    failed |= check_func(power(10, 4), 10000, "int power(10, 4)");
    failed |= check_func(power(10, 1),    10, "int power(10, 1)");
    failed |= check_func(power( 2, 8),   256, "int power( 2, 8)");
    failed |= check_func(factorial(4),    24, "int factorial(4)");
    failed |= check_func(factorial(1),     1, "int factorial(1)");
    failed |= check_func(factorial(0),     1, "int factorial(0)");
        
    failed |= check_monetary("32.40", 3240);
    failed |= check_monetary("4000.01", 400001);
    failed |= check_monetary("90.4", 9040);
    failed |= check_monetary("0.40", 40);
    failed |= check_monetary(".10", 10);
    failed |= check_monetary("0.99", 99);
    failed |= check_monetary("48.99", 4899);
    failed |= check_monetary("123", 12300);
    failed |= check_monetary("1", 100);
    failed |= check_monetary("10010", 1001000);
    failed |= check_monetary("-45.13", -4513);
    failed |= check_monetary("-1234", -123400);
    failed |= check_monetary("-1234.01", -123401);
    failed |= check_monetary("-1234.20", -123420);
    
    money amount1("320.40");
    money amount2("4");
    money amount3;
    failed |= check_money(amount1, 32040, "amount1 == 320.40");
    failed |= check_money(amount2, 400, "amount2 == 4");
    failed |= check_money(amount1+amount2, 32440, "amount1+amount2");
    failed |= check_money(amount1-amount2, 31640, "amount1-amount2");
    failed |= check_money(amount1/amount2, 8010, "amount1/amount2");
    failed |= check_money(amount1*amount2, 128160, "amount1*amount2");
    failed |= check_money(amount3, 0, "initial value is zero");
    failed |= check_money(amount3 = "20.01", 2001, "char * assignment");
    failed |= check_money(amount3 = std::string("123.5"), 12350, "std::string assignment");
    failed |= check_money(amount3 += money("1.1"), 12460, "+= assignment");
    failed |= check_money(amount3 -= "4.34", 12026, "-= assignment");
    failed |= check_money(amount3 *= money("2"), 24052, "*= assignment");
    failed |= check_money(amount3 /= money("2"), 12026, "/= assignment");
    amount3.neg();
    failed |= check_money(amount3, -12026, "neg()");
    
    std::ostringstream ss;
    money amount4("1234.05");
    money amount5("1234.50");
    
    ss << amount4;
    failed |= check_string(ss.str(), "1234.05", "output to ostream");

    failed |= check_func(amount4.trunc(), 1234, "trunc(1234.05)");
    failed |= check_func(amount2.trunc(), 4, "trunc(4)");
    failed |= check_func(amount4.frac(), 5, "frac(1234.05)");
    failed |= check_func(amount5.frac(), 50, "frac(1234.50)");

    return failed;
        
  }OAPR_PRINT(&std::cerr,return 1);
  return 0;
}
